@@ -123,12 +123,12 @@ rmd_RCM_fleet_output <- function(ff, f_name) {
            "  vul_bb[[bb]] <- sapply(report_list, function(x) if (!is.null(x$vul_len)) x$vul_len[, bl[bb]] else NA)",
            paste0("  Year_legend[bb] <- Year[RCMdata@sel_block[, ", ff, "] == bl[bb]] %>% range() %>% paste(collapse = \"-\")"),
            "}",
-           "test <- vapply(vul_bb, function(x) all(!is.na(x)), logical(1))",
-           "if (all(test)) {",
+           "test <- vapply(vul_bb, function(x) all(!is.na(x)) && sum(x), logical(1))",
+           "if (any(test)) {",
            paste0("  matplot(RCMdata@Misc$lbinmid, RCMdata@Misc$lbinmid, type = \"n\", xlab = \"Length\", ylim = c(0, 1), ylab = \"Selectivity of ", f_name[ff], "\")"),
            "  abline(h = 0, col = \"grey\")",
            "  for(bb in 1:length(bl)) {",
-           "    matlines(RCMdata@Misc$lbinmid, vul_bb[[bb]], type = \"l\", col = bl_col[bb], lty = scenario$lty, lwd = scenario$lwd)",
+           "    if (test[bb]) matlines(RCMdata@Misc$lbinmid, vul_bb[[bb]], type = \"l\", col = bl_col[bb], lty = scenario$lty, lwd = scenario$lwd)",
            "  }",
            "  if (length(bl) > 1) legend(\"topright\", Year_legend, col = bl_col, lwd = 1)",
            #"if (!is.null(scenario$names)) legend("topleft", scenario$names, col = scenario$col2, lty = scenario$lty, lwd = scenario$lwd)",
@@ -239,8 +239,8 @@ rmd_RCM_index_output <- function(sur, s_name) {
            "",
            paste0("```{r, fig.cap = \"Length selectivity of ", s_name[sur], ".\"}"),
            "ivul_len <- sapply(report_list, function(x) if (!is.null(x$ivul_len)) x$ivul_len[, ", sur, "] else NA)",
-           "if (all(!is.na(ivul_len))) {",
-           paste0("  matplot(RCMdata@Misc$lbinmid, ivul_len, type = \"n\", xlab = \"Length\", ylim = c(0, 1), ylab = \"Selectivity of ", s_name[sur], "\")"),
+           "if (all(!is.na(ivul_len)) && sum(ivul_len, na.rm = TRUE)) {",
+           paste0("  matplot(RCMdata@Misc$lbinmid, ivul_len, type = \"l\", xlab = \"Length\", ylim = c(0, 1), ylab = \"Selectivity of ", s_name[sur], "\")"),
            "  abline(h = 0, col = \"grey\")",
            "  if (!is.null(scenario$names)) legend(\"topleft\", scenario$names, col = scenario$col2, lty = scenario$lty, lwd = scenario$lwd)",
            "}",
