@@ -77,7 +77,7 @@
 #'
 #' @return An object of class \linkS4class{RCModel} (see link for description of output).
 #' 
-#' \code{check_RCMdata} returns a list of updated RCMdata object, OM, and StockPars, ObsPars, and FleetPars from the Hist object generated
+#' \code{check_RCMdata} returns a list of updated RCMdata object, OM, and StockPars and FleetPars from the Hist object generated
 #' from the OM.
 #'
 #' @section Online Documentation:
@@ -86,7 +86,7 @@
 #' \itemize{
 #' \item \href{https://openmse.com/tutorial-rcm/}{General overview of approach}
 #' \item \href{https://openmse.com/tutorial-rcm-eq/}{Mathematical description}
-#' \item \href{https://openmse.com/tutorial-rcm-select/}{Setup of selectivity settings} (useful for more data-rich cases)
+#' \item \href{https://openmse.com/tutorial-rcm-select/}{Setup of selectivity settings and index catchability} (useful for more data-rich cases)
 #' \item \href{https://openmse.com/features-assessment-models/5-priors/}{Description of priors}
 #' }
 #'
@@ -171,7 +171,7 @@
 #' the row specifies the selectivity at age. 
 #' \item log_rec_dev: A numeric vector of length nyears for the starting values of the log-recruitment deviations.
 #' \item log_early_rec_dev: A numeric vector of length OM@@maxage for the starting values of the recruitment deviations controlling the abundance-at-age in the first year of the model.
-#' \item q: A numeric vector of length nsurvey for index catchability.
+#' \item q: A numeric vector of length nsurvey for index catchability. See \href{https://openmse.com/tutorial-rcm-select/}{online article} for more information.
 #' }
 #' 
 #' Parameters can be fixed with the map argument (also a named list, corresponding to the start list). Each
@@ -187,7 +187,7 @@
 #' \item log_rec_dev: A vector of length OM@@nyears that indexes which recruitment deviates are fixed (using NA) or estimated (a separate integer).
 #' By default, all these deviates are estimated.
 #' \item q: A vector of length nsurvey for index catchability. q should be an estimated parameter when sharing across surveys (perhaps with differing selectivity). Otherwise, it is solved analytically
-#' where individual parameters are independent of other indices. Use \code{RCMdata@abs_I} for fixing the catchability to 1.
+#' where individual parameters are independent of other indices. Use \code{RCMdata@abs_I} for fixing the catchability to 1. See \href{https://openmse.com/tutorial-rcm-select/}{online article} for more information.
 #' }
 #' 
 #' @section Likelihood weights:
@@ -390,6 +390,7 @@ setMethod("RCM", signature(OM = "OM", data = "Data"),
             if (!is.null(Ind$Index)) {
               dataS4@Index <- Ind$Index
               dataS4@I_sd <- Ind$I_sd
+              if (is.null(Ind$I_sd)) stop("No standard errors for the index was found.")
 
               if (any(!is.na(Ind$V))) {
                 extra_args$ivul_par <- Ind$V
