@@ -152,9 +152,10 @@ report <- function(Assessment, retro = NULL, filename = paste0("report_", Assess
   }
   render_args$quiet <- quiet
 
-  message("Rendering markdown file...")
+  message("Rendering markdown file: ", file.path(dir, filename_rmd))
   output_filename <- do.call(rmarkdown::render, render_args)
   message("Rendered file: ", output_filename)
+  message("See help(plot.Assessment) to adjust report and file directory.")
 
   if (open_file) browseURL(output_filename)
   invisible(output_filename)
@@ -626,7 +627,7 @@ plot_residuals <- function(Year, res, res_sd = NULL, res_sd_CI = 0.95,
 #' @param fit A matrix of predicted length or age composition from an assessment model.
 #' Same dimensions as obs.
 #' @param plot_type Indicates which plots to create. Options include annual distributions,
-#' bubble plot of the data, and bubble plot of the residuals, and annual means.
+#' bubble plot of the data, and bubble plot of the Pearson residuals, and annual means.
 #' @param N Annual sample sizes. Vector of length \code{nrow(obs)}.
 #' @param CAL_bins A vector of lengths corresponding to the columns in \code{obs}.
 #' and \code{fit}. Ignored for age data.
@@ -640,18 +641,52 @@ plot_residuals <- function(Year, res, res_sd = NULL, res_sd_CI = 0.95,
 #' @param fit_linewidth Argument \code{lwd} for fitted line.
 #' @param fit_color Color of fitted line.
 #' @param bubble_color Colors for negative and positive residuals, respectively, for bubble plots.
-#' @return Plots depending on \code{plot_type}.
+#' @return Plots depending on \code{plot_type}. Invisibly returns a matrix or list of values that were plotted.
 #' @author Q. Huynh
 #' @export
 #' @examples
 #' plot_composition(obs = SimulatedData@@CAA[1, 1:16, ])
-#' plot_composition(obs = SimulatedData@@CAA[1, , ], plot_type = "bubble_data", 
-#'                  ages = 0:SimulatedData@@MaxAge)
+#' plot_composition(
+#'   obs = SimulatedData@@CAA[1, , ], 
+#'   plot_type = "bubble_data", 
+#'   ages = 0:SimulatedData@@MaxAge
+#' )
+#'                  
+#' SCA_fit <- SCA(x = 2, Data = SimulatedData)
+#' plot_composition(
+#'   obs = SimulatedData@CAA[1, , ], fit = SCA_fit@C_at_age,
+#'   plot_type = "mean", ages = 0:SimulatedData@MaxAge
+#' )
 #' 
-plot_composition <- function(Year = 1:nrow(obs), obs, fit = NULL, plot_type = c('annual', 'bubble_data', 'bubble_residuals', 'mean'),
+#' plot_composition(
+#'   obs = SimulatedData@CAA[1, 1:16, ], fit = SCA_fit@C_at_age[1:16, ],
+#'   plot_type = "annual", ages = 0:SimulatedData@MaxAge
+#' )
+#' 
+#' plot_composition(
+#'   obs = SimulatedData@CAA[1, , ], fit = SCA_fit@C_at_age,
+#'   plot_type = "bubble_residuals", ages = 0:SimulatedData@MaxAge
+#' )
+#' 
+#' plot_composition(
+#'   obs = SimulatedData@CAA[1, , ], fit = SCA_fit@C_at_age,
+#'   plot_type = "heat_residuals", ages = 0:SimulatedData@MaxAge
+#' )
+#' 
+#' plot_composition(
+#'   obs = SimulatedData@CAA[1, , ], fit = SCA_fit@C_at_age,
+#'   plot_type = "hist_residuals", ages = 0:SimulatedData@MaxAge
+#' )
+#' 
+#' @importFrom gplots redblue
+#' @importFrom graphics rect
+plot_composition <- function(Year = 1:nrow(obs), obs, fit = NULL, 
+                             plot_type = c('annual', 'bubble_data', 'bubble_residuals', 'mean', 'heat_residuals', 'hist_residuals'),
                              N = rowSums(obs), CAL_bins = NULL, ages = NULL, ind = 1:nrow(obs),
                              annual_ylab = "Frequency", annual_yscale = c("proportions", "raw"),
-                             bubble_adj = 5, bubble_color = c("black", "white"), fit_linewidth = 3, fit_color = "red") {
+                             bubble_adj = 1.5, #ifelse(plot_type == 'bubble_data', 5, 1.5), 
+                             bubble_color = c("#99999999", "white"), # grDevices::gray(0.6, 0.6)
+                             fit_linewidth = 3, fit_color = "red") {
   plot_type <- match.arg(plot_type)
   annual_yscale <- match.arg(annual_yscale)
   if (is.null(CAL_bins)) data_type <- "age" else data_type <- "length"
@@ -666,7 +701,8 @@ plot_composition <- function(Year = 1:nrow(obs), obs, fit = NULL, plot_type = c(
     data_val <- if (is.null(ages)) 1:ncol(obs) else ages
     data_lab <- "Age"
   }
-  if (!is.null(N)) N <- round(N, 1)
+  if (is.null(N)) N <- rowSums(obs)
+  N <- round(N, 1)
 
   if (annual_yscale == "proportions") {
     obs_prob_all <- obs/rowSums(obs, na.rm = TRUE)
@@ -687,51 +723,87 @@ plot_composition <- function(Year = 1:nrow(obs), obs, fit = NULL, plot_type = c(
   if (!is.null(N)) N <- N[ind]
 
   # Bubble plot (obs)
-  if ('bubble_data' %in% plot_type) {
+  if (plot_type == 'bubble_data') {
     range_obs <- pretty(obs, n = 6)
     n1 <- range_obs[2]
     n2 <- pretty(quantile(obs[obs > 0], na.rm = TRUE, probs = 0.9))[2]
     if (n2 < n1) n1 <- 0.5 * n2
     diameter_max <- bubble_adj / n2
+    
+    Year_mat <- matrix(Year, ncol = ncol(obs), nrow = nrow(obs))
+    data_mat <- matrix(data_val, ncol = ncol(obs), nrow = nrow(obs), byrow = TRUE)
+    
+    bubble_size <- bubble_adj * sqrt(pmin(obs * 3 / n2, 3))
     plot(NULL, NULL, typ = 'n', xlim = range(Year), xlab = "Year",
          ylim = range(data_val), ylab = data_lab)
-    for(i in 1:length(Year)) {
-      for(j in 1:length(data_val)) {
-        points(Year[i], data_val[j], cex = 0.5 * diameter_max * pmin(obs[i, j], n2), pch = 21, bg = "white")
-      }
+    points(Year_mat, data_mat, cex = bubble_size, pch = 21, bg = "white", col = "black")
+    
+    if (data_lab == "Age") {
+      lapply(pretty(c(min(Year) - max(data_val), max(Year))), function(x) abline(a = -x, b = 1, col = "gray50", lty = "dotted"))
     }
-    lapply(pretty(c(min(Year) - max(data_val), max(Year))), function(x) abline(a = -x, b = 1, col = "gray50", lty = "dotted"))
-    legend("topleft", legend = c(n1, paste0(">", n2)), pt.cex = 0.5 * diameter_max * c(n1, n2),
+    legend("topleft", legend = c(n1, paste0(">", n2)), pt.cex = bubble_adj * sqrt(c(n1, n2) * 3 / n2),
            pt.bg = "white", pch = 21, horiz = TRUE)
-    return(invisible())
+    return(invisible(obs))
   }
-  # Bubble plot (residuals if applicable)
-  if ('bubble_residuals' %in% plot_type) {
+  # Pearson residuals
+  if (grepl('residuals', plot_type)) {
     if (is.null(fit)) stop("No fitted data available.")
 
     obs_prob <- obs/rowSums(obs, na.rm = TRUE)
     fit_prob <- fit/rowSums(fit, na.rm = TRUE)
 
-    resid <- N * (obs_prob - fit_prob) / sqrt(N * fit_prob * (1 - fit_prob))
-    diameter_max <- bubble_adj / pmin(10, max(abs(resid), na.rm = TRUE))
-    plot(NULL, NULL, typ = 'n', xlim = range(Year), xlab = "Year", ylim = range(data_val), ylab = data_lab)
-
+    resid <- (obs_prob - fit_prob) / sqrt(fit_prob * (1 - fit_prob)/N)
+    max_abs_resid <- 3
+    
     Year_mat <- matrix(Year, ncol = ncol(resid), nrow = nrow(resid))
     data_mat <- matrix(data_val, ncol = ncol(resid), nrow = nrow(resid), byrow = TRUE)
-    isPositive <- resid > 0
-    points(Year_mat[!isPositive], data_mat[!isPositive], cex = pmin(0.5 * diameter_max * abs(resid[!isPositive]), diameter_max), pch = 21, bg = bubble_color[1])
-    points(Year_mat[isPositive], data_mat[isPositive], cex = pmin(0.5 * diameter_max * resid[isPositive], diameter_max), pch = 21, bg = bubble_color[2])
     
-    lapply(pretty(c(min(Year) - max(data_val), max(Year))), function(x) abline(a = -x, b = 1, col = "gray50", lty = "dotted"))
+    if (!grepl("hist", plot_type)) {
+      plot(NULL, NULL, typ = 'n', xlim = range(Year), xlab = "Year", ylim = c(1, 1.1) * range(data_val), ylab = data_lab)
+      if (data_type == "age") {
+        lapply(pretty(c(min(Year) - max(data_val), max(Year))), function(x) abline(a = -x, b = 1, col = "gray50", lty = "dotted"))
+      }
+    }
     
-    legend("topleft", legend = c("<-10", "-1", "1", ">10"),
-           pt.cex = c(diameter_max, 0.5 * diameter_max, 0.5 * diameter_max, diameter_max),
-           pt.bg = rep(bubble_color, each = 2), pch = 21, horiz = TRUE)
-
-    return(invisible())
+    if (plot_type == "bubble_residuals") {
+      cex_bubble <- bubble_adj * sqrt(pmin(abs(resid), Inf))
+      isPositive <- resid > 0
+      points(Year_mat[!isPositive], data_mat[!isPositive], cex = cex_bubble[!isPositive], 
+             pch = 21, bg = bubble_color[1])
+      points(Year_mat[isPositive], data_mat[isPositive], cex = cex_bubble[isPositive],
+             pch = 21, bg = bubble_color[2])
+      legend("topleft", 
+             legend = c(0.1, 0.5, 1, ">3"),
+             pt.cex = sqrt(c(0.1, 0.5, 1, 3)) * bubble_adj,
+             pt.bg = bubble_color[1],
+             pch = 21, horiz = TRUE)
+      
+    } else if (plot_type == "heat_residuals") {
+      
+      resid_round <- pmin(resid, max_abs_resid) %>% pmax(resid, -max_abs_resid) %>% round(2)
+      vals <- unique(as.numeric(resid_round)) %>% sort()
+      ncolor <- length(vals)
+      cols_redblue <- gplots::redblue(ncolor) %>% structure(names = vals)
+      ydiff <- local({
+        dd <- apply(data_mat, 1, diff)
+        rbind(dd, dd[nrow(dd), ]) %>% t()
+      })
+      
+      rect(xleft = Year_mat - 0.5, xright = Year_mat + 0.5,
+           ybottom = data_mat - 0.5 * ydiff, ytop = data_mat + 0.5 * ydiff, 
+           border = "grey60", col = cols_redblue[as.character(resid_round)])
+      legend("topleft", 
+             legend = c(paste0("<-", round(max_abs_resid, 1)), "0", paste0(">", round(max_abs_resid, 1))),
+             col = "grey60", pt.cex = 1, pt.bg = cols_redblue[c(1, floor(0.5 * ncolor), ncolor)], 
+             pch = 22, horiz = TRUE)
+      
+    } else if (plot_type == "hist_residuals") {
+      hist(as.numeric(resid), xlab = "Pearson residuals", main = NULL)
+    }
+    return(invisible(resid))
   }
   # Mean length or age over time
-  if ('mean' %in% plot_type) {
+  if (plot_type == 'mean') {
     mu <- mupred <- numeric(length = length(Year))
     for(i in 1:length(mu)) {
       mu[i] <- weighted.mean(data_val, obs[i, ], na.rm = TRUE)
@@ -741,11 +813,11 @@ plot_composition <- function(Year = 1:nrow(obs), obs, fit = NULL, plot_type = c(
     plot(Year[ind2], mu[ind2], xlab = "Year", ylab = paste0("Mean ", data_type), typ = "o")
     if (!is.null(fit)) lines(Year[ind2], mupred[ind2], lwd = fit_linewidth, col = fit_color)
 
-    return(invisible())
+    return(invisible(mu))
   }
 
   # Annual comps (obs vs. fitted if available)
-  if ("annual" %in% plot_type) {
+  if (plot_type == "annual") {
     old_par <- par(no.readonly = TRUE)
     on.exit(par(old_par))
     par(mfcol = c(4, 4), mar = rep(0, 4), oma = c(5.1, 5.1, 2.1, 2.1))
@@ -780,7 +852,7 @@ plot_composition <- function(Year = 1:nrow(obs), obs, fit = NULL, plot_type = c(
         mtext(annual_ylab, side = 2, line = 3.5, outer = TRUE)
       }
     }
-    return(invisible())
+    return(invisible(list(obs = obs_prob, fit = fit_prob)))
 
   }
 
